import re
from .nttt_logging import log_replacement
from .utilities import apply_to_every_other_part


def trim_md_tags(md_file_content, logging):
    return apply_to_every_other_part(md_file_content, "```", __trim_md_tags, logging)


def __trim_md_tags(md_file_content, logging):
    lines = md_file_content.split("\n")
    trimmed_lines = []
    for i in range(len(lines)):
        line = lines[i]
        star_count = line.count("*")

        is_list_item = star_count % 2 == 1 and line.lstrip().startswith("*")
        if not is_list_item:
            trimmed_lines.append(__trim_md_tags_in_line(line, logging))
        else:
            star_index = line.index("*")
            next_after_star_index = star_index + 1

            list_marker = line[:next_after_star_index]
            list_item_text = line[next_after_star_index:]

            trimmed_lines.append(list_marker + __trim_md_tags_in_line(list_item_text, logging))

    return '\n'.join(trimmed_lines)


def __trim_md_tags_in_line(s, logging):
    return re.sub(r'(?P<tag>`|_{1,3}|\*{1,3})(?P<content>.+?)(?P=tag)', replacement_builder(logging), s)


def replacement_builder(logging):
    def internal_replacement_builder(matchobj):
        original_text = matchobj.group()
        tag_name = matchobj.group("tag")
        stripped_content = matchobj.group("content").strip()

        replacement_text = "{}{}{}".format(tag_name, stripped_content, tag_name)
        log_replacement(original_text, replacement_text, logging)
        return replacement_text

    return internal_replacement_builder



class GeneralConstants:
    FILE_NAME_META_YML = "meta.yml"
    VOLUNTEER_ACKNOWLEDGEMENTS_CSV = (
        "https://docs.google.com/spreadsheets/d/e/"
        "2PACX-1vS_oJDVBL9f0YQxISmtcCp8V1Hf4zjw1BwIhm-w5GMsvA8Y4VdT9JAUSueRXI5me5usLjblEPsNlqIH/"
        "pub?gid=1717165158&single=true&output=csv")


class ArgumentKeyConstants:
    # Keys for the arguments dictionary
    INPUT = 'INPUT'
    OUTPUT = 'OUTPUT'
    ENGLISH = 'ENGLISH'
    LANGUAGE = 'LANGUAGE'
    VOLUNTEERS = 'VOLUNTEERS'
    FINAL = 'FINAL'
    DISABLE = 'DISABLE'
    LOGGING = 'LOGGING'
    YES = 'YES'


class RegexConstants:
    # Matches various spaces (but no newlines). The list is taken from http://jkorpela.fi/chars/spaces.html
    SPACES = " \t\u00A0\u1680\u180E\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u200B" \
                  "\u202F\u205F\u3000\uFEFF"
    COLONS = ":："
    QUOTES = '"”“«»'



import re
from .nttt_logging import log_replacement
from .constants import RegexConstants

pattern_blank = re.compile(rf'^_[{RegexConstants.SPACES}]+blank$')


def trim_formatting_tags(md_file_content, logging):
    s = f"[{RegexConstants.SPACES}]"
    c = f"[{RegexConstants.COLONS}]"
    q = f"[{RegexConstants.QUOTES}]"
    return re.sub(rf'(?P<last_word>\S+?){s}*{{{s}*{c}{s}*(?P<tag>[\w]+?){s}*={s}*{q}{s}*(?P<value>.+?){s}*{q}{s}*}}',
                  replacement_builder(logging),
                  md_file_content)


def replacement_builder(logging):
    def internal_replacement_builder(matchobj):
        original_text = matchobj.group()
        last_word = matchobj.group("last_word")
        tag_name = matchobj.group("tag").lower()
        value = matchobj.group("value").lower()

        if pattern_blank.match(value):
            value = "_blank"

        replacement_text = '{}{{:{}="{}"}}'.format(last_word, tag_name, value)
        log_replacement(original_text, replacement_text, logging)
        return replacement_text

    return internal_replacement_builder



from nttt import main, tidyup_translations

#tidyup_translations("C:\\Users\\Martin O'Hanlon-LT\\Documents\\temp\\coordinates\\", "C:\\Users\\Martin O'Hanlon-LT\\Documents\\temp\\coordinates\\fixed\\")
main()


from .constants import ArgumentKeyConstants
import os
from pathlib import Path
from argparse import ArgumentParser


def get_absolute_path(folder):
    '''
    Returns the absolute path for the given folder. Trailing path separators
    and double quotes are also removed.
    '''

    folder = folder.strip().rstrip(os.pathsep).rstrip('"')
    folder = Path(folder).absolute()
    return folder


def get_step_file(folder, step):
    '''Returns the step file for the given step number.'''

    return Path(folder, "step_" + str(step) + ".md")


def get_final_step(folder):
    '''
    Returns the number of the final step file, or 0 in case there are no step
    files.
    '''

    final_step = 0
    step = 1
    step_file = get_step_file(folder, step)
    while os.path.isfile(step_file):
        final_step = step
        step += 1
        step_file = get_step_file(folder, step)
    return final_step


def parse_command_line(version):
    """
    Parses the command line and returns the arguments provided on command line.
    The convention is to begin functional flag names with a small letter
    and non-functional flag names with a capital letter
    """

    parser = ArgumentParser(description="Nina's Translation Tidyup Tool v{}".format(version))
    parser.add_argument("-i", "--input",      help="The input directory which contains the content to tidy up, defaults to the current directory.")
    parser.add_argument("-o", "--output",     help="The output directory where the upgraded content should be written, defaults to the same as INPUT.")
    parser.add_argument("-e", "--english",    help="The directory which contains the English files and folders, defaults to INPUT/../en.")
    parser.add_argument("-l", "--language",   help="The language of the content to be tidied up, defaults to basename(INPUT).")
    parser.add_argument("-v", "--volunteers", help="The list of volunteers as a comma separated list, defaults to an empty list.")
    parser.add_argument("-f", "--final",      help="The number of the final step file, defaults to the step file with the highest number.")
    parser.add_argument("-D", "--Disable",    help="The risky features to be disabled, separated by commas. "
                                                   "Options are: fix_md (fix common markdown-related issues), "
                                                   "fix_html (fix common issues in HTML-like tags (<kbd>Return</kbd>)), "
                                                   "fix_sections (fix common issues in section tags (--- hint ---)), "
                                                   "revert_section_translation (revert translation for section tags), "
                                                   "fix_formatting (fix common issues in formatting tags ({:class=\"block3motion\"})). "
                                                   "Defaults to all risky features to be enabled.")
    parser.add_argument("-L", "--Logging",    help="Logging of modifications. Options are on and off. Default is off.")
    parser.add_argument("-Y", "--Yes",        help="Automatic yes to prompts. "
                                                   "If enabled assume 'yes' as answer to all prompts and run non-interactively. "
                                                   "Options are on and off. Default is off.")
    return parser.parse_args()


def resolve_arguments(command_line_args):
    '''
    Returns the complete set of arguments. For arguments that are not provided
    on the command line, the default is used.
    '''

    from os.path import basename, dirname

    arguments = {}

    if command_line_args.input:
        arguments[ArgumentKeyConstants.INPUT] = get_absolute_path(command_line_args.input)
    else:
        arguments[ArgumentKeyConstants.INPUT] = get_absolute_path('.')

    if command_line_args.output:
        arguments[ArgumentKeyConstants.OUTPUT] = get_absolute_path(command_line_args.output)
    else:
        arguments[ArgumentKeyConstants.OUTPUT] = arguments[ArgumentKeyConstants.INPUT]

    if command_line_args.english:
        arguments[ArgumentKeyConstants.ENGLISH] = get_absolute_path(command_line_args.english)
    else:
        arguments[ArgumentKeyConstants.ENGLISH] = Path(dirname(arguments[ArgumentKeyConstants.INPUT]), 'en')

    if command_line_args.language:
        arguments[ArgumentKeyConstants.LANGUAGE] = command_line_args.language
    else:
        arguments[ArgumentKeyConstants.LANGUAGE] = basename(arguments[ArgumentKeyConstants.INPUT])

    if command_line_args.volunteers:
        arguments[ArgumentKeyConstants.VOLUNTEERS] = [name.strip() for name in command_line_args.volunteers.split(',')]
    else:
        arguments[ArgumentKeyConstants.VOLUNTEERS] = []

    if command_line_args.final:
        arguments[ArgumentKeyConstants.FINAL] = int(command_line_args.final)
    else:
        arguments[ArgumentKeyConstants.FINAL] = get_final_step(arguments[ArgumentKeyConstants.INPUT])

    if command_line_args.Disable:
        arguments[ArgumentKeyConstants.DISABLE] = command_line_args.Disable.split(",")
    else:
        arguments[ArgumentKeyConstants.DISABLE] = []

    if command_line_args.Logging:
        arguments[ArgumentKeyConstants.LOGGING] = command_line_args.Logging
    else:
        arguments[ArgumentKeyConstants.LOGGING] = "off"

    if command_line_args.Yes:
        arguments[ArgumentKeyConstants.YES] = command_line_args.Yes
    else:
        arguments[ArgumentKeyConstants.YES] = "off"

    return arguments


def show_arguments(arguments):
    '''Shows the given arguments.'''

    if arguments[ArgumentKeyConstants.INPUT] == arguments[ArgumentKeyConstants.OUTPUT]:
        print("Using folder - {}".format(arguments[ArgumentKeyConstants.INPUT]))
    else:
        print("Input folder - '{}'".format(arguments[ArgumentKeyConstants.INPUT]))
        print("Output folder - '{}'".format(arguments[ArgumentKeyConstants.OUTPUT]))
    print("English folder - '{}'".format(arguments[ArgumentKeyConstants.ENGLISH]))
    print("Language - '{}'".format(arguments[ArgumentKeyConstants.LANGUAGE]))
    print("Volunteers - '{}'".format(arguments[ArgumentKeyConstants.VOLUNTEERS]))
    print("Final step - '{}'".format(arguments[ArgumentKeyConstants.FINAL]))
    print("Disabled functions - '{}'".format(arguments[ArgumentKeyConstants.DISABLE]))
    print("Logging - '{}'".format(arguments[ArgumentKeyConstants.LOGGING]))
    print("Yes - '{}'".format(arguments[ArgumentKeyConstants.YES]))


def check_folder(folder):
    '''Checks whether the given folder exists and is a directory.'''

    import sys

    valid = True
    if not os.path.isdir(folder):
        valid = False
        print("Folder '{}' not found".format(folder), file=sys.stderr)
    return valid


def check_step_file(folder, step):
    '''Checks whether the step file for the given number exists.'''

    import sys

    valid = True
    step_file = get_step_file(folder, step)
    if not os.path.isfile(step_file):
        valid = False
        print('Step file {} not found.'.format(step_file), file=sys.stderr)
    return valid


def check_arguments(arguments):
    '''Checks whether the given arguments are valid.'''

    valid = True
    if not check_folder(arguments[ArgumentKeyConstants.INPUT]):
        valid = False
    if not check_folder(arguments[ArgumentKeyConstants.ENGLISH]):
        valid = False
    if os.path.exists(arguments[ArgumentKeyConstants.OUTPUT]) and not check_folder(arguments[ArgumentKeyConstants.OUTPUT]):
        valid = False
    if arguments[ArgumentKeyConstants.FINAL] > 0 and not check_step_file(arguments[ArgumentKeyConstants.INPUT], arguments[ArgumentKeyConstants.FINAL]):
        valid = False
    return valid



def log_replacement(text_before, text_after, logging):
    if logging == "on" and text_before != text_after:
        print("Replaced: {}".format(text_before))
        print("    with: {}".format(text_after))



import re
import sys
from .nttt_logging import log_replacement
from .constants import RegexConstants


section_tag_name_regex = f'/?\\w+(?:[\\-{RegexConstants.SPACES}]\\w+)*'


def fix_sections(md_file_content, logging):
    # For some weird reason Crowdin replaces '---' to '\---' in its output. So let's revert it back
    md_file_content = md_file_content.replace("\\---", "---")

    s = f"[{RegexConstants.SPACES}]"

    # Fixes 2 issues:
    # - users could mistakenly remove one dash
    # - users could mistakenly remove spaces around the tag
    md_file_content = re.sub(rf'---?{s}*(?P<tag>{section_tag_name_regex}?){s}*---?',
                             replacement_builder(logging, ["tag"], "--- {} ---"),
                             md_file_content)

    # Removes any possible spaces between '/' and tag name in closing tags
    md_file_content = re.sub(rf'--- /{s}+(?P<tag>{section_tag_name_regex}?) ---',
                             replacement_builder(logging, ["tag"], "--- /{} ---"),
                             md_file_content)

    # For some weird reason Crowdin jams 'hints' and 'hint' tags into one line it its output.
    # So "--- hints ---\n--- hint ---" becomes "--- hints --- --- hint ---".
    # Probably because they go in adjacent lines (no empty line between them).
    # So let's revert it back (also considering situations when translators mistakenly
    # modified those strings, for example translated them in target language
    md_file_content = re.sub(rf'--- (?P<tag>{section_tag_name_regex}?) ---{s}+(?=.+)',
                             replacement_builder(logging, ["tag"], "--- {} ---\n"),
                             md_file_content)
    md_file_content = re.sub(rf'(?P<content>\S+?){s}+--- (?P<tag>{section_tag_name_regex}?) ---',
                             replacement_builder(logging, ["content", "tag"], "{}\n--- {} ---"),
                             md_file_content)

    # For some weird reason Crowdin breaks 'title' tags. So let's revert it back
    c = f"[{RegexConstants.COLONS}]"
    md_file_content = re.sub(rf'##\n--- collapse ---\n\n## (?P<tag>.+?){c}{s}*(?P<title>.+?)\n',
                             replacement_builder(logging, ["title"], "--- collapse ---\n---\ntitle: {}\n---\n"),
                             md_file_content)

    return md_file_content


def replacement_builder(logging, groups_to_extract, replacement_pattern):
    def internal_replacement_builder(matchobj):
        original_text = matchobj.group()
        group_values = [matchobj.group(group_name) for group_name in groups_to_extract]

        replacement_text = replacement_pattern.format(*group_values)
        log_replacement(original_text, replacement_text, logging)
        return replacement_text

    return internal_replacement_builder


def revert_section_translation(md_file_name, md_file_content, en_file_content, logging):
    """
    Compares translated file to the original one
    and reverts translated section tags ("--- opdracht ---" -> "--- task ---").
    This happens if both the original file and the translated one contain the same number of those tags
    """
    section_pattern = re.compile("--- (?P<tag>.+?) ---")

    md_file_lines = md_file_content.split('\n')
    md_file_section_dict = extract_sections(md_file_lines, section_pattern)

    en_file_lines = en_file_content.split('\n')
    en_file_section_dict = extract_sections(en_file_lines, section_pattern)

    if len(md_file_section_dict) == len(en_file_section_dict):
        md_sec_line_num = sorted(md_file_section_dict)
        en_sec_line_num = sorted(en_file_section_dict)

        for i in range(len(md_sec_line_num)):
            idx_section_to_replace = md_sec_line_num[i]
            replacement_text = en_file_section_dict[en_sec_line_num[i]]

            log_replacement(md_file_lines[idx_section_to_replace], replacement_text, logging)
            md_file_lines[idx_section_to_replace] = replacement_text

        return '\n'.join(md_file_lines)
    else:
        print("Warning ({}): Different section structure in the original (en) and the translated pages. "
              "Reverting of translated section tags will not be performed".format(md_file_name), file=sys.stderr)
        return md_file_content


# return dictionary of the following structure:
# line number -> section text (i. e.: 0 -> "--- hints ---")
def extract_sections(md_file_lines, section_pattern):
    sections = {}

    for i in range(len(md_file_lines)):
        line = md_file_lines[i]
        if section_pattern.match(line):
            sections[i] = line

    return sections



from .arguments import parse_command_line, resolve_arguments, check_arguments, show_arguments
from .tidyup import tidyup_translations
from ._version import __version__

def main():
    command_line_args = parse_command_line(__version__)
    resolved_arguments = resolve_arguments(command_line_args)
    show_arguments(resolved_arguments)
    if (check_arguments(resolved_arguments)):
        tidyup_translations(resolved_arguments)



import pandas
import sys
from .utilities import get_file, save_file


# In case of an empty spreadsheet cell, pandas will return a float value NaN.
# In string format, this becomes 'nan'.
EMPTY_CELL_AS_STRING = 'nan'


def get_volunteer_acknowledgement(csv_file_path, language):
    '''
    Returns the volunteer acknowledgement for the given language or None if
    the volunteer acknowledgement doesn't exist.
    '''

    data_frame = None
    try:
        data_frame = pandas.read_csv(csv_file_path)
    except:
        print("Could not read the volunteer acknowledgements spreadsheet.",
              file=sys.stderr)
        return None

    for i in data_frame.index:
        if data_frame.at[i, 'Locale'] == language:
            return data_frame.at[i, 'Acknowledgement']
    return None


def add_volunteer_acknowledgement(csv_file_path, output_file_path, language,
                                  volunteers, logging):
    '''
    Adds the volunteer acknowledgement to the given output file. Returns
    whether the volunteer acknowledgement was successfully added.
    '''

    if logging == "on":
        print("Using the volunteer acknowledgements from {}".format(csv_file_path))

    acknowledgement = get_volunteer_acknowledgement(csv_file_path, language)
    if acknowledgement is None:
        print("Could not find the volunteer acknowledgement for language {}".format(language),
              file=sys.stderr)
        return False
    elif str(acknowledgement) == EMPTY_CELL_AS_STRING:
        print("Empty volunteer acknowledgement for language {}".format(language),
              file=sys.stderr)
        return False

    name_placeholders = "[name]\n\n[name]\n\n[name]\n"
    if name_placeholders not in acknowledgement:
        print("Incorrect volunteer acknowledgement for language {}".format(language),
              file=sys.stderr)
        return False

    content, suggested_eol = get_file(output_file_path)
    if "***\n" in content:
        print("Volunteer acknowledgement already present in file {}".format(output_file_path),
              file=sys.stderr)
        return False

    if len(volunteers) == 0:
        print("Warning: No volunteer name(s) given - please add them manually")
    else:
        volunteer_names = '\n'.join(volunteers) + '\n'
        acknowledgement = acknowledgement.replace(name_placeholders,
                                                  volunteer_names)
    if logging == "on":
        print("Volunteer acknowledgement to be added:")
        print(acknowledgement)

    # Strip trailings blanks (spaces, tabs, newlines) from the original content
    # before appending the volunteer acknowledgement. This avoids a surplus of
    # empty lines between the content and the volunteer acknowledgement.
    content = content.rstrip() + "\n\n***\n" + acknowledgement + "\n"
    save_file(output_file_path, content, suggested_eol)
    return True



import re
from .nttt_logging import log_replacement
from .utilities import apply_to_every_other_part


def trim_html_tags(md_file_content, logging):
    # The idea of this loop is to detect text that goes inside backquotes (`) and keep that text unchanged.
    # This is because in Markdown files backquotes enclose code snippets that usually should be not be changed.
    # We achieve this by splitting the content by backquotes and applying html trimming to every other part.
    # Due to rather lucky coincidence this code also takes care of not changing content inside triple backquotes (```)
    return apply_to_every_other_part(md_file_content, "`", __trim_html_tags, logging)


def __trim_html_tags(s, logging):
    return re.sub(r'<(?P<tag>[\w\d]+?)>(?P<content>.+?)</(?P=tag)>', replacement_builder(logging), s)


def replacement_builder(logging):
    def internal_replacement_builder(matchobj):
        original_text = matchobj.group()
        tag_name = matchobj.group("tag")
        stripped_content = matchobj.group("content").strip()

        replacement_text = "<{}>{}</{}>".format(tag_name, stripped_content, tag_name)
        log_replacement(original_text, replacement_text, logging)
        return replacement_text

    return internal_replacement_builder



import os.path
from pathlib import Path
from shutil import copyfile, copytree
import eol


def get_file(file_name):
    with open(file_name, encoding='utf-8') as f:
        (_, suggested_eol) = eol.eol_info_from_path(file_name)
        return f.read(), suggested_eol


def save_file(file_name, content, suggested_eol):
    with open(file_name, encoding='utf-8', mode="w", newline=suggested_eol) as f:
        f.write(content)


def find_files(src, file_names=[], extensions=[]):
    files_found = []

    for dname, dirs, files in os.walk(src):
        for fname in files:
            fpath = os.path.join(dname, fname)

            # check file_names and extensions
            valid_file = False

            if len(extensions) > 0:
                ext = os.path.splitext(fpath)
                if ext[-1] in extensions:
                    valid_file = True

            if len(file_names) > 0:
                if fname in file_names:
                    valid_file = True

            # if we arent checking file_names or extension it must be valid
            if len(extensions) == 0 and len(file_names) == 0:
                valid_file = True

            if valid_file:
                files_found.append(fpath)

    return sorted(files_found)


def find_missing_entries(source_folder, dest_folder):
    '''
    Returns a list of folders that are present in the source folder, but not in
    the destination folder plus a list of files that are present in the source
    folder, but not in the destination folder.
    Files or folders that are inside a folder from the missing folders list are
    not present in the missing files and folders lists.
    '''

    missing_folders = []
    missing_files = []
    for item in source_folder.iterdir():
        base_item = os.path.basename(item)
        dest_item = Path(dest_folder, base_item)
        if not dest_item.exists():
            if item.is_dir():
                missing_folders.append(base_item)
            elif item.is_file():
                missing_files.append(base_item)

    return (missing_folders, missing_files)


def copy_missing_folders(source_folder, folders_to_copy, dest_folder):
    '''
    Copies the given folders, including contents, from the source folder to the
    destination folder.
    '''
    _copy_missing_items("folder", source_folder, folders_to_copy, copytree, dest_folder)


def copy_missing_files(source_folder, files_to_copy, dest_folder):
    '''
    Copies the given files from the source folder to the destination folder.
    '''
    _copy_missing_items("file", source_folder, files_to_copy, copyfile, dest_folder)


def _copy_missing_items(type_of_items, source_folder, items_to_copy, copy_function, dest_folder):
    '''
    Copies the given items from the source folder to the destination folder,
    using the provided copy function.
    '''

    print("Copying {}s from {} to {}:".format(type_of_items, source_folder, dest_folder))
    for item_to_copy in items_to_copy:
        print(" - {}".format(item_to_copy))

    for item_to_copy in items_to_copy:
        source_path = Path(source_folder, item_to_copy)
        dest_path = Path(dest_folder, item_to_copy)
        copy_function(source_path, dest_path)


def add_missing_entries(input_folder, english_folder, output_folder):
    '''
    Adds missing files and folders to the output folder.

    If the output folder is not the same as the input folder, all files and
    folders in the input folder that are not in the output folder, will be
    copied to the output folder.
    Typically, this will be the images, resources and solutions folders,
    because previous processing steps will have only resulted in a meta.yml
    file and mark-down files in the output folder.

    All folders in the English folder that are not in the output folder, will
    also be copied to the output folder.
    Any file in the English folder that is not in the output folder, will NOT
    be copied to the output folder. The user will have to do this manually.
    '''

    # Start with the input folder, if it is not the same as the output folder.
    if input_folder != output_folder:
        (folders_to_copy, files_to_copy) = find_missing_entries(input_folder, output_folder)
        if len(folders_to_copy) > 0:
            copy_missing_folders(input_folder, folders_to_copy, output_folder)
        if len(files_to_copy) > 0:
            copy_missing_files(input_folder, files_to_copy, output_folder)

    # Next is the English folder; only missing folders should be copied,
    # missing files should not be copied.
    (folders_to_copy, files_to_copy) = find_missing_entries(english_folder, output_folder)
    if len(folders_to_copy) > 0:
        copy_missing_folders(english_folder, folders_to_copy, output_folder)


def apply_to_every_other_part(content, separator, func, param1):
    """
    Splits the content by the given separator
    and applies the given function to every other part (0th, 2nd, 4th and so on)
    keeping the rest (1st, 3rd, 5th and so on) intact
    """
    parts = content.split(separator)
    processed_parts = []

    for i in range(len(parts)):
        part = parts[i]
        if (i % 2) == 0:
            processed_parts.append(func(part, param1))
        else:
            processed_parts.append(parts[i])

    return separator.join(processed_parts)


import sys
import os
from setuptools import setup

if sys.version_info[0] == 2:
    raise ValueError('This package requires Python 3.7 or newer')
elif sys.version_info[0] == 3:
    if not sys.version_info >= (3, 7):
        raise ValueError('This package requires Python 3.7 or newer')
else:
    raise ValueError('Unrecognized major version of Python')

# Read version
version_globals = {}
with open(os.path.join("nttt", "_version.py")) as fp:
    exec(fp.read(), version_globals)

__project__ = 'nttt'
__desc__ = 'A utility for Nina to clean up translated projects'
__version__ = version_globals['__version__']
__author__ = "Martin O'Hanlon"
__author_email__ = 'martin.ohanlon@raspberrypi.org'
__url__ = 'https://github.com/raspberrypilearning/nttt'

if __name__ == '__main__':
    setup(name='nttt',
          version = __version__,
          description = __desc__,
          url = __url__,
          author = __author__,
          author_email = __author_email__,
          packages = [__project__],
          entry_points={
              'console_scripts': [
                  'nttt = nttt:main'
              ]},
          zip_safe=False)



import os
import io
import ruamel.yaml
from .acknowledgements import add_volunteer_acknowledgement
from .arguments import get_step_file
from .constants import ArgumentKeyConstants, GeneralConstants
from .utilities import add_missing_entries, find_files, get_file, save_file
from .cleanup_markdown import trim_md_tags
from .cleanup_html import trim_html_tags
from .cleanup_formatting import trim_formatting_tags
from .cleanup_sections import fix_sections
from .cleanup_sections import revert_section_translation


def fix_meta(src, english_src, dst):
    (content, suggested_eol) = get_file(src)
    (english_content, _) = get_file(english_src)

    # Fix steps
    content = content.replace("  - \n    title:", "  - title:")

    # Revert untranslatable elements
    content = revert_untranslatable_meta_elements(content, english_content)

    save_file(dst, content, suggested_eol)


def revert_untranslatable_meta_elements(content, english_content):
    yaml_parser = ruamel.yaml.YAML(typ='rt')
    yaml_parser.preserve_quotes = True
    # Disable timestamp parsing. If enabled it may lead to unwanted errors when the date is invalid.
    # Beware that this approach relies on not documented features. Some details:
    # https://stackoverflow.com/questions/50900727/skip-converting-entities-while-loading-a-yaml-string-using-pyyaml
    yaml_parser.constructor.yaml_constructors.pop(u'tag:yaml.org,2002:timestamp', None)

    parsed_md = yaml_parser.load(content)
    english_parsed_md = yaml_parser.load(english_content)

    translatable_keys = ["title", "description", "steps"]
    for key in parsed_md:
        if key not in translatable_keys and key in english_parsed_md:
            parsed_md[key] = english_parsed_md[key]

    yaml_dumper = ruamel.yaml.YAML()
    yaml_dumper.indent(sequence=4, offset=2)
    yaml_dumper.explicit_start = True
    yaml_dumper.width = 1000000

    string_buffer = io.StringIO()
    yaml_dumper.dump(parsed_md, string_buffer)
    string_buffer.seek(0)
    return string_buffer.read()


def fix_md_step(src, lang, english_src, dst, disable, logging):
    (md_content, suggested_eol) = get_file(src)
    en_md_content = None
    if os.path.isfile(english_src):
        (en_md_content, _) = get_file(english_src)

    if "fix_sections" not in disable:
        md_content = fix_sections(md_content, logging)
        if en_md_content is not None and "revert_section_translation" not in disable:
            md_content = revert_section_translation(src, md_content, en_md_content, logging)

    if "fix_md" not in disable:
        md_content = trim_md_tags(md_content, logging)

    if "fix_html" not in disable:
        md_content = trim_html_tags(md_content, logging)

    if "fix_formatting" not in disable:
        md_content = trim_formatting_tags(md_content, logging)

    # update language in urls
    md_content = md_content.replace("/en/", "/" + lang + "/")

    save_file(dst, md_content, suggested_eol)


def tidyup_translations(arguments):
    input_folder = arguments[ArgumentKeyConstants.INPUT]
    output_folder = arguments[ArgumentKeyConstants.OUTPUT]
    english_folder = arguments[ArgumentKeyConstants.ENGLISH]
    language = arguments[ArgumentKeyConstants.LANGUAGE]
    disable = arguments[ArgumentKeyConstants.DISABLE]
    logging = arguments[ArgumentKeyConstants.LOGGING]
    volunteers = arguments[ArgumentKeyConstants.VOLUNTEERS]
    final_step = arguments[ArgumentKeyConstants.FINAL]
    yes = arguments[ArgumentKeyConstants.YES]

    # get files to update
    print("Find files ...")
    files_to_update = find_files(input_folder, file_names=[GeneralConstants.FILE_NAME_META_YML], extensions=[".md"])

    if len(files_to_update) > 0:
        print("About to tidy up files:")
        for file in files_to_update:
            print(" - {}".format(os.path.relpath(file, input_folder)))

        continue_with_cleanup = True
        if yes != 'on':
            process_yn = input("Continue (y/n):")
            continue_with_cleanup = (process_yn.casefold() == "y")

        if continue_with_cleanup:
            for source_file_path in files_to_update:
                relative_input_file_name = os.path.relpath(source_file_path, input_folder)

                output_file_path = os.path.join(output_folder, relative_input_file_name)

                # create output folder
                output_file_folder = os.path.dirname(output_file_path)

                if not os.path.exists(output_file_folder):
                    os.makedirs(output_file_folder)

                print("Fixing - {}".format(relative_input_file_name))
                en_file_path = os.path.join(english_folder, relative_input_file_name)
                if relative_input_file_name == GeneralConstants.FILE_NAME_META_YML:
                    fix_meta(source_file_path, en_file_path, output_file_path)
                else:
                    fix_md_step(source_file_path, language, en_file_path, output_file_path, disable, logging)

            if final_step > 0:
                output_file_path = get_step_file(output_folder, final_step)
                print("Adding volunteer acknowledgement - {}".format(
                        output_file_path))
                add_volunteer_acknowledgement(
                    GeneralConstants.VOLUNTEER_ACKNOWLEDGEMENTS_CSV,
                    output_file_path, language, volunteers, logging)

            print("Complete")

    else:
        print("No files found in '{}'".format(input_folder))

    # add files and folders missing in the output folder
    add_missing_entries(input_folder, english_folder, output_folder)
